package eu.mhsl.craftattack.spawn.appliances.event;

import eu.mhsl.craftattack.spawn.appliance.Appliance;
import eu.mhsl.craftattack.spawn.appliance.ApplianceCommand;
import eu.mhsl.craftattack.spawn.appliances.event.command.EventCommand;
import eu.mhsl.craftattack.spawn.appliances.event.command.EventEndSessionCommand;
import eu.mhsl.craftattack.spawn.appliances.event.command.EventOpenSessionCommand;
import eu.mhsl.craftattack.spawn.appliances.event.command.MoveEventVillagerCommand;
import eu.mhsl.craftattack.spawn.util.DisplayVillager;
import eu.mhsl.craftattack.spawn.util.PluginMessage;
import eu.mhsl.craftattack.spawn.util.commonListeners.DismissInventoryOpenFromHolder;
import eu.mhsl.craftattack.spawn.util.commonListeners.PlayerInteractAtEntityEventListener;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.entity.Player;
import org.bukkit.entity.Villager;
import org.bukkit.event.Listener;
import org.jetbrains.annotations.NotNull;

import java.util.List;

public class Event extends Appliance {
    public DisplayVillager.ConfigBound villager;
    private boolean isOpen = false;

    public Event() {
        super("event");
    }

    @Override
    public void onEnable() {
        this.villager = new DisplayVillager.ConfigBound(
                localConfig(),
                villager -> {
                    villager.customName(Component.text("Events", NamedTextColor.GOLD));
                    villager.setProfession(Villager.Profession.LIBRARIAN);
                    villager.setVillagerType(Villager.Type.SNOW);
                }
        );
    }

    public void openEvent() {
        if(isOpen) throw new ApplianceCommand.Error("Es läuft derzeit bereits ein Event!");
        isOpen = true;
    }

    public void joinEvent(Player p) {
        if(!isOpen) {
            p.sendMessage(Component.text("Zurzeit ist kein Event geöffnet.", NamedTextColor.RED));
            return;
        }

        PluginMessage.connect(p, localConfig().getString("connect-server-name"));
    }

    public void endEvent() {
        if(!isOpen) throw new ApplianceCommand.Error("Es läuft derzeit kein Event!");
        isOpen = false;
    }

    @Override
    protected @NotNull List<ApplianceCommand<?>> commands() {
        return List.of(
                new EventCommand(),
                new MoveEventVillagerCommand(),
                new EventOpenSessionCommand(),
                new EventEndSessionCommand()
        );
    }

    @Override
    protected @NotNull List<Listener> eventHandlers() {
        return List.of(
                new PlayerInteractAtEntityEventListener(this.villager.getUniqueId(), playerInteractAtEntityEvent -> joinEvent(playerInteractAtEntityEvent.getPlayer())),
                new DismissInventoryOpenFromHolder(this.villager.getUniqueId())
        );
    }
}
